

#include "stdio.h"
#include <string.h>

#include "xutil.h"
#include "xuartlite_l.h"
#include "xemaclite.h"

#include "mb_interface.h"
#include "xparameters.h"
#include "xintc.h"
#include "xps2.h"

#include "../stage2boot/gfxlib.c"

#define MY_MAC_ADDR "\x02\x00\x22\x00\x00\xf0"

static XIntc intc;
static XEmacLite emac;
static XPs2 ps2;

static void Keyboard_ISR (void *CallBackRef, u32 IntrMask, u32 NumBytes)
{
    u8          key;

    if (!(IntrMask & XPS2_IPIXR_RX_FULL)) {
        return;
    }
    while (XPs2_IsRxFull(XPAR_PS2_0_BASEADDR)) {
        key = XPs2_RecvByte(XPAR_PS2_0_BASEADDR);
        printf ("key %02x\n", key);
    }
}

static void Ethernet_Receive_ISR (void *CallBackRef)
{
    u8 data[1600];
    u16 size;
    u16 i;
    
    size = XEmacLite_Recv(&emac, data);
    if (size == 0) return;

    /* Receiving a packet, print if leftmost switch is on */
    Recache_Buttons ();
    if (shm[ SWITCHES ] & 0x1000) {
        printf ("packet %d: ", size);
        for (i = 0; (i < size) && (i < 30); i ++) {
            printf ("%02x", data[ i ]);
        }
        printf ("\n");
    }
}

static void Ethernet_Send_ISR (void *CallBackRef)
{
    /* printf ("send complete\n"); */
}

int main (void)
{
    XEmacLite_Config *  emaccfg;
    XPs2_Config *       ps2cfg;
    int                 rc;
    unsigned            last_centre_press, centre_press;

    printf("Initialising (1)...\n");

    /* Patch block RAM so that interrupts are handled by this
     * program rather than the boot ROM */
    memcpy ( (void *) 0x8 , 
            (void *) ( (unsigned) sdram + 0x8 ) , 0x18 ) ; 

    /* Clear graphics */
    memset(video, 0, 64000);

    /* Begin initialising peripherals */
    emaccfg = XEmacLite_LookupConfig (XPAR_EMACLITE_0_DEVICE_ID);
    assert (emaccfg != NULL);
    ps2cfg = XPs2_LookupConfig (XPAR_PS2_0_DEVICE_ID);
    assert (ps2cfg != NULL);

    rc = XEmacLite_CfgInitialize(&emac, emaccfg, XPAR_EMACLITE_0_BASEADDR);
    assert (rc == XST_SUCCESS);
    rc = XPs2_CfgInitialize(&ps2, ps2cfg, XPAR_PS2_0_BASEADDR);
    assert (rc == XST_SUCCESS);

    microblaze_register_handler (
        (XInterruptHandler) XIntc_InterruptHandler,
        &intc );
    rc = XIntc_Initialize(&intc, XPAR_XPS_INTC_0_DEVICE_ID);
    assert ((rc == XST_SUCCESS) || (rc == XST_DEVICE_IS_STARTED));

    rc = XIntc_Connect( &intc,
            XPAR_INTC_0_EMACLITE_0_VEC_ID,
            (XInterruptHandler)XEmacLite_InterruptHandler, &emac);
    assert (rc == XST_SUCCESS);
    rc = XIntc_Connect( &intc,
            XPAR_INTC_0_PS2_0_VEC_ID,
            (XInterruptHandler)XPs2_IntrHandler, &ps2);
    assert (rc == XST_SUCCESS);

    rc = XIntc_Start ( &intc, XIN_REAL_MODE);
    assert (rc == XST_SUCCESS);

    printf("Initialising (2)...\n");

    XPs2_SetHandler(&ps2, Keyboard_ISR, NULL);
    XPs2_IntrGlobalEnable(&ps2);
    XPs2_IntrEnable(&ps2, XPS2_IPIXR_RX_ALL);
    XEmacLite_SetMacAddress(&emac, (const u8 *) MY_MAC_ADDR);
    XEmacLite_FlushReceive(&emac);
    XEmacLite_SetRecvHandler(&emac, NULL, Ethernet_Receive_ISR);
    XEmacLite_SetSendHandler(&emac, NULL, Ethernet_Send_ISR);
    XEmacLite_EnableInterrupts(&emac);

    rc = XEmacLite_SelfTest(&emac);
    assert (rc == XST_SUCCESS);

    printf("Initialising (3)...\n");

    XIntc_Enable (&intc, XPAR_INTC_0_PS2_0_VEC_ID );
    XIntc_Enable (&intc, XPAR_INTC_0_EMACLITE_0_VEC_ID );
    microblaze_enable_interrupts();

        
    printf("System ready\n");
    printf("Press centre button to send a packet.\n"
            "Turn on the left-most switch to see broadcast packets.\n");
    last_centre_press = centre_press = shm[ BTN_CENTRE ];

    while (1) {
        Recache_Buttons ();
        centre_press = shm[ BTN_CENTRE ];
        
        if (centre_press != last_centre_press) {
            const char packet[] =
                    "\xff\xff\xff\xff\xff\xff"  /* destination: broadcast */
                    MY_MAC_ADDR                 /* source: me */
                    "\x12\x34"                  /* protocol */
                    "we apologise for the inconvenience!\0\0"; /* payload */

            unsigned size = sizeof(packet) & ~1 ; /* note: size rounded down */

            rc = XEmacLite_Send(&emac, packet, size);
            printf("send: rc = %d size = %u\n", rc, size);

            last_centre_press = centre_press;
        }
    }
    return 0;
}


